#!/usr/bin/env python3
import click
from slacktools.auth import get_slackclient_config
from slacktools.post import post_slack_message
from slacktools.react import post_slack_reaction
from slacktools.unpost import delete_slack_message
from slacktools.unreact import delete_slack_reaction
from slacktools.channel import channel_set_topic, channel_user_add, channel_user_delete, channel_create, channel_archive, channel_unarchive
from toolbox.logger import Log
from toolbox.click_complete import complete_emojis
from toolbox import misc

MESSAGE="Slack CLI Client" + misc.MOVE + "Current Profile: " + misc.GREEN + misc.UNDERLINE + 'DEFAULT' + misc.RESET

@click.group(help=MESSAGE, context_settings={'help_option_names':['-h','--help'], 'max_content_width': misc.set_terminal_width()}, invoke_without_command=True)
@click.option('-p', '--profile', help='User profile to use for connecting to Slack', required=False, default='default', type=str)
@click.pass_context
def slack(ctx, profile):
    ctx.ensure_object(dict)
    ctx.obj['PROFILE'] = profile
    pass

@slack.command(help='post a meessage to a given channel(s)', context_settings={'help_option_names':['-h','--help'], 'max_content_width': misc.set_terminal_width()})
@click.argument('channel', nargs=-1, type=str, required=True)
@click.option('-m', '--message', help='Text to include in the message', required=True, type=str)
@click.option('-r', '--reply', help='TS of the message to reply to', required=False, type=str)
@click.pass_context
def post(ctx, channel, message, reply):
    profile = ctx.obj['PROFILE']
    RESPONSES = post_slack_message(channel, message, reply, profile)
    for RESPONSE in RESPONSES:
        Log.info(f"Message sent at {RESPONSE['ts']}")

@slack.command(help='post a reaction to a given message in specific channel', context_settings={'help_option_names':['-h','--help'], 'max_content_width': misc.set_terminal_width()})
@click.argument('channel', nargs=-1, type=str, required=True)
@click.option('-t', '--timestamp', help='timestamp of the target message', required=True, type=str)
@click.option('-e', '--emoji', help='emoji name to react with', required=True, type=str, shell_complete=complete_emojis)
@click.pass_context
def react(ctx, channel, timestamp, emoji):
    profile = ctx.obj['PROFILE']
    post_slack_reaction(channel, timestamp, emoji, profile)
    Log.info("Reaction posted")

@slack.command(help='delete a meessage in a given channel', context_settings={'help_option_names':['-h','--help'], 'max_content_width': misc.set_terminal_width()})
@click.argument('channel', nargs=-1, type=str, required=True)
@click.option('-t', '--timestamp', help='Text to include in the message', required=True, type=str)
@click.pass_context
def unpost(ctx, channel, timestamp):
    profile = ctx.obj['PROFILE']
    delete_slack_message(channel, timestamp, profile)
    Log.info(f"Message deleted")

@slack.command(help='remove a reaction to a given message in a specific channel', context_settings={'help_option_names':['-h','--help'], 'max_content_width': misc.set_terminal_width()})
@click.argument('channel', nargs=-1, type=str, required=True)
@click.option('-t', '--timestamp', help='timestamp of the target message', required=True, type=str)
@click.option('-e', '--emoji', help='emoji name to react with', required=True, type=str)
@click.pass_context
def unreact(ctx, channel, timestamp, emoji):
    profile = ctx.obj['PROFILE']
    delete_slack_reaction(channel, timestamp, emoji, profile)
    Log.info("Reaction removed")

@click.group(help='manage Slack channels; create, archive, invite users etc', context_settings={'help_option_names':['-h','--help'], 'max_content_width': misc.set_terminal_width()})
@click.pass_context
def channel(ctx):
    pass

@channel.command('create', help='create a new slack channel', context_settings={'help_option_names':['-h','--help'], 'max_content_width': misc.set_terminal_width()})
@click.option('-n', '--name', help='name for the new channel', required=False, type=str)
@click.option('-p', '--private', help='should the new channel be private', required=False, default=False, type=bool)
@click.pass_context
def create(ctx, name, private):
    profile = ctx.obj['PROFILE']
    channel_create(name,private, profile)

@channel.command('adduser', help='add (invite) new user to the channel', context_settings={'help_option_names':['-h','--help'], 'max_content_width': misc.set_terminal_width()})
@click.option('-i', '--id', help='target channel', required=False, type=str)
@click.option('-u', '--user', help='user id to invite to the channel', required=False, type=str)
@click.pass_context
def adduser(ctx, id, user):
    profile = ctx.obj['PROFILE']
    channel_user_add(id,user, profile)

@channel.command('deluser', help='delete (kick) a user from the channel', context_settings={'help_option_names':['-h','--help'], 'max_content_width': misc.set_terminal_width()})
@click.option('-i', '--id', help='target channel', required=False, type=str)
@click.option('-u', '--user', help='user id to kick from the channel', required=False, type=str)
@click.pass_context
def deluser(ctx, id, user):
    profile = ctx.obj['PROFILE']
    channel_user_delete(id,user, profile)

@channel.command('archive', help='set channel status to "archived"', context_settings={'help_option_names':['-h','--help'], 'max_content_width': misc.set_terminal_width()})
@click.option('-i', '--id', help='target channel', required=False, type=str)
@click.pass_context
def archive(ctx, id):
    profile = ctx.obj['PROFILE']
    channel_archive(id, profile)

@channel.command('unarchive', help='remove "archived" status from channel', context_settings={'help_option_names':['-h','--help'], 'max_content_width': misc.set_terminal_width()})
@click.option('-i', '--id', help='target channel', required=False, type=str)
@click.pass_context
def unarchive(ctx, id):
    profile = ctx.obj['PROFILE']
    channel_unarchive(id, profile)

@channel.command('topic', help='set the topic message for the channel', context_settings={'help_option_names':['-h','--help'], 'max_content_width': misc.set_terminal_width()})
@click.option('-i', '--id', help='target channel', required=False, type=str)
@click.option('-t', '--topic', help='what to put in the topic channel', required=False, type=str)
@click.pass_context
def topic(ctx, id, topic):
    profile = ctx.obj['PROFILE']
    channel_set_topic(id, topic, profile)

@slack.command(help="retrieve the entire content of slacktools's configstore", context_settings={'help_option_names':['-h','--help'], 'max_content_width': misc.set_terminal_width()})
@click.pass_context
def config(ctx):
    RESULT = get_slackclient_config()
    return RESULT

slack.add_command(post)
slack.add_command(react)
slack.add_command(unpost)
slack.add_command(unreact)
slack.add_command(channel)



import click, json, os
from toolbox.logger import Log
from configstore.configstore import Config
from .presets.jenkins import jenkins_preset
from toolbox.misc import set_terminal_width, detect_environment

@click.group(help='manage built-in and custom presets', context_settings={'help_option_names':['-h','--help'], 'max_content_width': set_terminal_width()})
@click.option('-d', '--debug', help="0 = no output, 1 = default, 2 = debug on", default='1', type=click.Choice(['0', '1', '2']))
def preset(debug):
    log = Log('goat.log', debug)
    pass

@preset.command(help='add new data:key pairs to presets', context_settings={'help_option_names':['-h','--help'], 'max_content_width': set_terminal_width()})
@click.argument('type', required=True, type=click.Choice(['config', 'metadata']))
@click.argument('module', required=True)
@click.argument('data_pairs', nargs=-1, type=str, required=True)
def add(type, module, data_pairs):
    for DATA_PAIR in data_pairs:
        DATA_PAIR = DATA_PAIR.split(':',1)
        KEY = DATA_PAIR[0]
        VALUE = DATA_PAIR[1]
        if type == 'config':
            RESULT = Config(module).update_config(VALUE, KEY, 'preset')
        elif type == 'metadata':
            RESULT = Config(module).update_metadata(VALUE, KEY, 'preset')
        else:
            RESULT = False
    return RESULT


#@preset.command(help='load built-in presets for enabled modules', context_settings={'help_option_names':['-h','--help'], 'max_content_width': set_terminal_width()})
def init():
    # put any tools in here that shouldn't use presets in prod
    if detect_environment() == 'non-gc':
        RESULT = Config('jenkinstools').create_preset(jenkins_preset)
    else:
        RESULT = False
    return RESULT

@preset.command(help='delete given keys from presets', context_settings={'help_option_names':['-h','--help'], 'max_content_width': set_terminal_width()})
@click.argument('type', required=True, type=click.Choice(['config', 'metadata']))
@click.argument('module', required=True)
@click.argument('key_names', nargs=-1, type=str, required=True)
def delete(type, module, key_names):
    for KEY in key_names:
        if type == 'config':
            RESULT = Config(module).clear_config(KEY, 'preset')
        elif type == 'metadata':
            RESULT = Config(module).clear_metadata(KEY, 'preset')
        else:
            RESULT =  False
    return RESULT

@preset.command(help='clear preset attached to a specific module', context_settings={'help_option_names':['-h','--help'], 'max_content_width': set_terminal_width()})
@click.argument('module')
def clear(module):
    RESULT = Config(module).clear_preset()
    return RESULT

@preset.command(help='load a preset from a given file and upload it to a specific module', context_settings={'help_option_names':['-h','--help'], 'max_content_width': set_terminal_width()})
@click.argument('module')
@click.argument('preset_file', type=click.File('rb'))
def load(module, preset_file):
    PRESET_DATA = json.loads(preset_file.read())
    RESULT = Config(module).create_preset(PRESET_DATA)
    return RESULT




#!/usr/bin/env python3

from prompt_toolkit.application import Application
import click, os, sys, operator
from pathlib import Path
from toolbox.logger import Log
from jiratools.jiraclient import cli as jira
from azdevops.azdev import cli as azdev
from configstore.configstore_ctrl import cli as configs
from awstools.awstools import CLI as aws
from aztools.aztools import CLI as az
from ovhaitools.ovhaitools import CLI as ovhai
from ibmcloudtools.ibmcloudtools import CLI as ibmcloud
from ocitools.ocitools import CLI as oci
from jenkinstools.jenkinstools import cli as jenkins
from toolbox.misc import set_terminal_width, search_man_pages, debug, draw_title
from .cmd_slack import slack

if os.getenv('USER') == 'root':
    print('...unable to run goat as root! Exiting')
    exit(1)

MESSAGE = 'goat => GCP, OCI, & AWS TK'

@click.group(help=MESSAGE, context_settings={'help_option_names':['-h','--help'], 'max_content_width': set_terminal_width()}, invoke_without_command=True)
@click.option('-v', '--version', help="print version of goat and all its submodules", default=False, is_flag=True)
@click.option('-m', '--manuals', help="print all defined manuals matching pattern(s)", type=str, multiple=True, required=False)
@click.option('-s', '--shell', help="launch the GOAT Shell for improved cloud interaction", default=False, is_flag=True, required=False)
@click.pass_context
def cli(ctx, version, manuals, shell):
    ctx.ensure_object(dict)
    if version:
        goat_version()
        sys.exit(0)
    if shell:
        import logging
        from goatshell.parser import Parser
        from goatshell.completer import GoatCompleter
        from goatshell.main import cli as Goatshell
        logger = logging.getLogger(__name__)
        Goatshell()
    if manuals:
        name = search_man_pages(manuals)
        os.system(f"man {name}")
        sys.exit(0)
    Log.setup('goat', int(debug))
    pass

def goat_version():
    from importlib_metadata import version
    RES = draw_title()
    print('GOAT:\t\t\t' + version('goaat'))
    print('- awstools:\t\t' + version('goatawstools'))
    print('- aztools:\t\t' + version('goataztools'))
    print('- azdevops:\t\t' + version('goatazdevops'))
    print('- ovhaitools:\t\t' + version('goatovhaitools'))
    print('- ibmcloudtools:\t' + version('goatibmcloudtools'))
    print('- configstore:\t\t' + version('goatconfigstore'))
    print('- jenkinstools:\t\t' + version('goatjenkinstools'))
    print('- jiratools:\t\t' + version('goatjiratools'))
    print('- ocitools:\t\t' + version('goatocitools'))
    print('- slacktools:\t\t' + version('goatslacktools'))
    print('- toolbox:\t\t' + version('goattoolbox'))
    print(RES)
    sys.exit(0)

cli.add_command(azdev, name='azdev')
cli.add_command(configs, name='configs')
cli.add_command(jira, name='jira')
cli.add_command(aws, name='amazon')
cli.add_command(az, name='azure')
cli.add_command(ibmcloud, name='ibm')
cli.add_command(oci, name='oracle')
cli.add_command(ovhai, name='ovh')
cli.add_command(jenkins, name='jenkins')
cli.add_command(slack)

if __name__ == "__main__":
    cli(ctx)

